package me.mrletsplay.shareclient.util.listeners;

import java.nio.charset.StandardCharsets;

import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentListener;

import me.mrletsplay.shareclient.ShareClient;
import me.mrletsplay.shareclient.util.ProjectAndPath;
import me.mrletsplay.shareclient.util.ProjectRelativePath;
import me.mrletsplay.shareclient.util.ShareSession;
import me.mrletsplay.shareclient.util.SharedProject;
import me.mrletsplay.shareclientcore.document.SharedDocument;

public class ShareClientDocumentListener implements IDocumentListener {

	private ProjectAndPath path;
	private IDocument document;

	private boolean ignoreChanges = false;

	public ShareClientDocumentListener(ProjectAndPath path, IDocument document) {
		this.path = path;
		this.document = document;
	}

	public void setIgnoreChanges(boolean ignoreChanges) {
		this.ignoreChanges = ignoreChanges;
	}

	public boolean isIgnoreChanges() {
		return ignoreChanges;
	}

	@Override
	public void documentAboutToBeChanged(DocumentEvent event) {
		ShareClient.assertOnUIThread();

		if(ignoreChanges) return;

		System.out.println("UPDATE ON THREAD " + Thread.currentThread());

		// FIXME: bug somewhere here, can cause discrepancies in local editor when editing while changes are coming in

		ShareSession session = ShareClient.getDefault().getActiveSession();
		if(session == null) return;

		SharedProject sharedProject = session.getSharedProject(path.project());
		if(sharedProject == null) return;

		SharedDocument doc = session.getOrCreateSharedDocument(new ProjectRelativePath(sharedProject.getRemoteName(), path.path()), () -> event.fDocument.get().getBytes(StandardCharsets.UTF_8));
		if(event.getLength() > 0) {
			doc.localDelete(event.getOffset(), event.getLength());
		}

		doc.localInsert(event.getOffset(), event.getText());
	}

	@Override
	public void documentChanged(DocumentEvent event) {

	}

	public IDocument getDocument() {
		return document;
	}

}
